{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "fi",
  "action.edit": "Muokkaa tätä sivua",
  "action.skip": "Hyppää sisältöön",
  "action.view": "Näytä tämän sivun lähdekoodi",
  "announce.dismiss": "Älä näytä tätä enää",
  "blog.archive": "Arkisto",
  "blog.categories": "Kategoriat",
  "blog.categories.in": "",
  "blog.continue": "Jatka lukemista",
  "blog.draft": "Luonnos",
  "blog.index": "Takaisin hakemistoon",
  "blog.meta": "Metatiedot",
  "blog.references": "Liittyvät linkit",
  "clipboard.copy": "Kopioi leikepöydälle",
  "clipboard.copied": "Kopioitu leikepöydälle",
  "consent.accept": "Hyväksy",
  "consent.manage": "Hallinnoi asetuksia",
  "consent.reject": "Hylkää",
  "footer": "Alatunniste",
  "footer.next": "Seuraava",
  "footer.previous": "Edellinen",
  "header": "Ylätunniste",
  "meta.comments": "Kommentit",
  "meta.source": "Lähdekodi",
  "nav": "Navigaatio",
  "readtime.one": "1 min lukuaika",
  "readtime.other": "# min lukuaika",
  "rss.created": "RSS-syöte",
  "rss.updated": "RSS-syöte päivitetylle sisällölle",
  "search": "Haku",
  "search.config.lang": "fi",
  "search.placeholder": "Hae",
  "search.share": "Jaa",
  "search.reset": "Tyhjää",
  "search.result.initializer": "Aloitetaan hakua",
  "search.result.placeholder": "Kirjoita aloittaaksesi haun",
  "search.result.none": "Ei täsmääviä dokumentteja",
  "search.result.one": "1 täsmäävä dokumentti",
  "search.result.other": "# täsmäävää dokumenttia",
  "search.result.more.one": "1 lisää tällä sivulla",
  "search.result.more.other": "# lisää tällä sivulla",
  "search.result.term.missing": "Puuttuu",
  "select.language": "Valitse kieli",
  "select.version": "Valitse versio",
  "source": "Mene repositoryyn",
  "source.file.contributors": "Avustajat",
  "source.file.date.created": "Luotu",
  "source.file.date.updated": "Viimeisin päivitys",
  "tabs": "Välilehdet",
  "toc": "Sisällysluettelo",
  "top": "Takaisin ylös"
}[key] }}{% endmacro %}
