{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "mn",
  "action.edit": "Хуудас засварлах",
  "action.skip": "Агуулгыг алгасах",
  "action.view": "Энэ хуудасны эх кодыг харах",
  "announce.dismiss": "Дахин үзүүлэхгүй",
  "blog.archive": "Архив",
  "blog.categories": "Ангилал",
  "blog.categories.in": "дотор",
  "blog.continue": "Үргэлжлүүлэн унших",
  "blog.draft": "Ноорог",
  "blog.index": "Жагсаалт руу буцах",
  "blog.meta": "Мета өгөгдөл",
  "blog.references": "Холбогдох холбоосууд",
  "clipboard.copy": "Хуулах",
  "clipboard.copied": "Хуулсан",
  "consent.accept": "Зөвшөөрөх",
  "consent.manage": "Тохиргоо удирдах",
  "consent.reject": "Татгалзах",
  "footer": "Хөл",
  "footer.next": "Дараах",
  "footer.previous": "Өмнөх",
  "header": "Гарчиг",
  "meta.comments": "Сэтгэгдэл",
  "meta.source": "Эх үүсвэр",
  "nav": "Чиглүүлэгч",
  "readtime.one": "1 минутын унших хугацаа",
  "readtime.other": "# минутын унших хугацаа",
  "rss.created": "RSS хангамж",
  "rss.updated": "Шинэчлэгдсэн агуулгын RSS хангамж",
  "search": "Хайх",
  "search.placeholder": "Хайлт",
  "search.share": "Хуваалцах",
  "search.reset": "Дахин эхлэх",
  "search.result.initializer": "Хайлтыг эхлүүлж байна",
  "search.result.placeholder": "Хайлтын үгээ бичнэ үү",
  "search.result.none": "Таарц илэрсэнгүй",
  "search.result.one": "1 таарц илэрлээ",
  "search.result.other": "# Тохирох баримт бичиг",
  "search.result.more.one": "1 илүү хуудас байна",
  "search.result.more.other": "# илүү хуудас байна",
  "search.result.term.missing": "Дутуу",
  "select.language": "Хэл сонгох",
  "select.version": "Хувилбар сонгох",
  "source": "Хадгалах сан руу очих",
  "source.file.contributors": "Хувь нэмэр оруулагчид",
  "source.file.date.created": "Үүсгэсэн",
  "source.file.date.updated": "Сүүлийн шинэчлэлт",
  "tabs": "Табууд",
  "toc": "Агуулга",
  "top": "Дээш буцах"
}[key] }}{% endmacro %}
