{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "sr",
  "action.edit": "Ажурирај страницу",
  "action.skip": "Иди на текст",
  "action.view": "Погледај изворни код ове странице",
  "announce.dismiss": "Не приказуј поново",
  "blog.archive": "Архива",
  "blog.categories": "Категорије",
  "blog.categories.in": "у",
  "blog.continue": "Настави са читањем",
  "blog.draft": "Нацрт",
  "blog.index": "Назад на индекс",
  "blog.meta": "Метаподаци",
  "blog.references": "Сродне везе",
  "clipboard.copy": "Копирај у clipboard",
  "clipboard.copied": "Ископирано у clipboard",
  "consent.accept": "Прихвати",
  "consent.manage": "Управљај подешавањима",
  "consent.reject": "Одбиј",
  "footer": "Подножје",
  "footer.next": "Следеће",
  "footer.previous": "Претходно",
  "header": "Заглавље",
  "meta.comments": "Коментари",
  "meta.source": "Извор",
  "nav": "Навигација",
  "readtime.one": "1 минут читања",
  "readtime.other": "# минута читања",
  "rss.created": "RSS feed",
  "rss.updated": "RSS feed ажурираног садржаја",
  "search": "Претрага",
  "search.placeholder": "Претрага",
  "search.share": "Дељење",
  "search.reset": "Очисти",
  "search.result.initializer": "Иницијализујем претрагу",
  "search.result.placeholder": "Унеси појам претраге",
  "search.result.none": "Ништа није пронађено",
  "search.result.one": "1 резултат претраге",
  "search.result.other": "# резултата претраге",
  "search.result.more.one": "још 1 на овој страници",
  "search.result.more.other": "још # на овој страници",
  "search.result.term.missing": "Недостаје",
  "select.language": "Изабери језик",
  "select.version": "Изабери верзију",
  "source": "Иди у репозиторијум",
  "source.file.contributors": "Сарадници",
  "source.file.date.created": "Креирано",
  "source.file.date.updated": "Ажурирано",
  "tabs": "Табови",
  "toc": "Садржај",
  "top": "Назад на врх"
}[key] }}{% endmacro %}
