{#-
  This file was automatically generated - do not edit
-#}
{% set class = "md-tag" %}
{% if tag.hidden %}
  {% set class = class ~ " md-tag-shadow" %}
{% endif %}
{% if config.extra.tags %}
  {% set class = class ~ " md-tag-icon" %}
  {% if tag.name in config.extra.tags %}
    {% set class = class ~ " md-tag--" ~ config.extra.tags[tag.name] %}
  {% endif %}
{% endif %}
<span class="{{ class }}">
  {{- tag.name -}}
</span>
