{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "pt",
  "action.edit": "Editar esta página",
  "action.skip": "Ir para o conteúdo",
  "action.view": "Ver fonte desta página",
  "announce.dismiss": "Não mostrar novamente",
  "blog.archive": "Arquivo",
  "blog.categories": "Categorias",
  "blog.categories.in": "em",
  "blog.continue": "Continuar leitura",
  "blog.draft": "Rascunho",
  "blog.index": "Voltar ao índice",
  "blog.meta": "Metadados",
  "blog.references": "Ligações relacionadas",
  "clipboard.copy": "Copiar para área de transferência",
  "clipboard.copied": "Copiado para área de transferência",
  "consent.accept": "Aceitar",
  "consent.manage": "Gerir configurações",
  "consent.reject": "Rejeitar",
  "footer": "Rodapé",
  "footer.next": "Próximo",
  "footer.previous": "Anterior",
  "header": "Cabeçalho",
  "meta.comments": "Comentários",
  "meta.source": "Fonte",
  "nav": "Navegação",
  "readtime.one": "1 min de leitura",
  "readtime.other": "# min de leitura",
  "rss.created": "canal RSS",
  "rss.updated": "canal RSS com conteúdo atualizado",
  "search": "Pesquisar",
  "search.config.lang": "pt",
  "search.placeholder": "Buscar",
  "search.share": "Compartilhar",
  "search.reset": "Limpar",
  "search.result.initializer": "Inicializando a pesquisa",
  "search.result.placeholder": "Digite para iniciar a busca",
  "search.result.none": "Nenhum resultado encontrado",
  "search.result.one": "1 resultado encontrado",
  "search.result.other": "# resultados encontrados",
  "search.result.more.one": "Mais 1 nesta página",
  "search.result.more.other": "Mais # nesta página",
  "search.result.term.missing": "Ausente",
  "select.language": "Selecione o idioma",
  "select.version": "Selecione a versão",
  "source": "Ir ao repositório",
  "source.file.contributors": "Colaboradores",
  "source.file.date.created": "Criada",
  "source.file.date.updated": "Última atualização",
  "tabs": "Abas",
  "toc": "Índice",
  "top": "Voltar ao topo"
}[key] }}{% endmacro %}
