{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "kn",
  "action.edit": "ಈ ಪುಟವನ್ನು ತಿದ್ದುಪಡಿ ಮಾಡಿ",
  "action.skip": "ವಿಷಯಕ್ಕೆ ತೆರಳಿ",
  "action.view": "ಈ ಪುಟದ ಮೂಲವನ್ನು ವೀಕ್ಷಿಸಿ",
  "announce.dismiss": "ಇದನ್ನು ಮತ್ತೊಮ್ಮೆ ತೋರಿಸಬೇಡಿ",
  "blog.archive": "ಹಳೆಯ ಲೇಖನ",
  "blog.categories": "ವರ್ಗಗಳು",
  "blog.categories.in": "ರಲ್ಲಿ",
  "blog.continue": "ಓದು ಮುಂದುವರೆಸಿ",
  "blog.draft": "ಆರಂಭಿಕ ಬರವಣಿಗೆ",
  "blog.index": "ಸೂಚ್ಯಂಕಕ್ಕೆ ಹಿಂತಿರುಗಿ",
  "blog.meta": "ಮಾಹಿತಿಯ ಬಗ್ಗೆ ಮಾಹಿತಿ",
  "blog.references": "ಸಂಬಂಧಿತ ಉಲ್ಲೇಖಗಳು",
  "clipboard.copy": "ಇದನ್ನು ನಕಲಿಸಿ",
  "clipboard.copied": "ಇದನ್ನು ನಕಲು ಮಾಡಿದೆ",
  "consent.accept": "ನಾನು ಇದನ್ನು ಒಪ್ಪಿಕೊಳ್ಳುತ್ತೇನೆ",
  "consent.manage": "ಸಂರಚನೆಯನ್ನು ನಿರ್ವಹಿಸಿ",
  "consent.reject": "ನಾನು ಇದನ್ನು ತಿರಸ್ಕರಿಸುತ್ತೇನೆ",
  "footer": "ಅಡಿಟಿಪ್ಪಣಿ",
  "footer.next": "ಮುಂದಿನ ಸಂಚಿಕೆ",
  "footer.previous": "ಹಿಂದಿನ ಸಂಚಿಕೆ",
  "header": "ಮೇಲ್ಟಿಪ್ಪಣಿ",
  "meta.comments": "ಪ್ರತಿಕ್ರಿಯೆಗಳು",
  "meta.source": "ಮೂಲ",
  "nav": "ಸಂಚರಣೆ",
  "readtime.one": "ಓದಲು ೧ ನಿಮಿಷ ತೆಗೆದುಕೊಳ್ಳುತ್ತದೆ",
  "readtime.other": "ಓದಲು # ನಿಮಿಷಗಳನ್ನು ತೆಗೆದುಕೊಳ್ಳುತ್ತದೆ",
  "rss.created": "ಆರ್ಎಸ್ಎಸ್ ಸೇವೆ",
  "rss.updated": "ಆರ್ಎಸ್ಎಸ್ ಸೇವೆಯಿಂದ ಇತ್ತೀಚಿನ ನವೀಕರಣ",
  "search": "ಹುಡುಕಿ",
  "search.placeholder": "ಹುಡುಕಿ",
  "search.share": "ಹಂಚಿಕೊಳ್ಳಿ",
  "search.reset": "ಅಳಿಸು",
  "search.result.initializer": "ಹುಡುಕಾಟವನ್ನು ಪ್ರಾರಂಭಿಸಲಾಗುತ್ತಿದೆ",
  "search.result.placeholder": "ಬರೆಯುವ ಮೂಲಕ ಹುಡುಕಲು ಪ್ರಾರಂಭಿಸಿ",
  "search.result.none": "ಹೊಂದಾಣಿಕೆಯಾಗುವ ದಾಖಲೆಗಳಿಲ್ಲ",
  "search.result.one": "೧ ಹೊಂದಾಣಿಕೆಯ ದಾಖಲೆಯಿದೆ",
  "search.result.other": "# ಹೊಂದಾಣಿಕೆಯ ದಾಖಲೆಗಳಿವೆ",
  "search.result.more.one": "ಈ ಪುಟದಲ್ಲಿ ಇನ್ನೂ ಒಂದು ಕಂಡುಬಂದಿದೆ",
  "search.result.more.other": "ಈ ಪುಟದಲ್ಲಿ ಇನ್ನೂ # ಇವೆ",
  "search.result.term.missing": "ಕಾಣೆಯಾಗಿದೆ",
  "select.language": "ಭಾಷೆಯನ್ನು ಆಯ್ಕೆಮಾಡಿ",
  "select.version": "ಆವೃತ್ತಿಯನ್ನು ಆಯ್ಕೆಮಾಡಿ",
  "source": "ಭಂಡಾರಕ್ಕೆ ಹೋಗಿ",
  "source.file.contributors": "ಕೊಡುಗೆದಾರರು",
  "source.file.date.created": "ರಚಿಸಿದ ದಿನಾಂಕ",
  "source.file.date.updated": "ಕೊನೆಯ ನವೀಕರಣ ದಿನಾಂಕ",
  "tabs": "ವಿವಿಧ ಕಿಟಕಿಗಳು",
  "toc": "ವಿಷಯಗಳ ಪಟ್ಟಿ",
  "top": "ಮೇಲಕ್ಕೆ ಹಿಂತಿರುಗಿ"
}[key] }}{% endmacro %}
