{#-
  This file was automatically generated - do not edit
-#}
{% import "partials/tabs-item.html" as item with context %}
<nav class="md-tabs" aria-label="{{ lang.t('tabs') }}" data-md-component="tabs">
  <div class="md-grid">
    <ul class="md-tabs__list">
      {% for nav_item in nav.items %}
        {{ item.render(nav_item) }}
      {% endfor %}
    </ul>
  </div>
</nav>
