{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "my",
  "action.edit": "ဤ စာမျက်နှာကို ပြင်ရန်",
  "action.skip": "အကြောင်းအရာသို့ ကရန်",
  "action.view": "ဤစာမျက်နှာ၏ ရင်းမြစ်ကို ကြည့်ရန်",
  "announce.dismiss": "နောက်တစ်ကြိမ် မပြတော့ပါ",
  "blog.archive": "မော်ကွန်းတိုက်",
  "blog.categories": "အမအစားများ",
  "blog.categories.in": "ထဲတွင်",
  "blog.continue": "ဆက်လက်ဖတ်ရှုရန်",
  "blog.draft": "မူကြမ်း",
  "blog.index": "အညွှန်းသို့ ပြန်သွားရန်",
  "blog.meta": "အချက်အလက်",
  "blog.references": "ဆက်စပ်လင့်ခ်များ",
  "clipboard.copy": "ကလစ်ဘုတ်သို့ ကူးယူရန်",
  "clipboard.copied": "ကလစ်ဘုတ်သို့ ကူးယူပြီးပါပြီ",
  "consent.accept": "လက်ခံသည်",
  "consent.manage": "ဆက်တင်များ စီမံရန်",
  "consent.reject": "ငြင်းပယ်သည်",
  "footer": "အောက်ခြေ",
  "footer.next": "ရှေ့သို့",
  "footer.previous": "နောက်သို့",
  "header": "ခေါင်းပိုင်း",
  "meta.comments": "မှတ်ချက်များ",
  "meta.source": "ရင်းမြစ်",
  "nav": "လမ်းညွှန်",
  "readtime.one": "၁ မိနစ် ဖတ်ရန်",
  "readtime.other": "မိနစ် # ဖတ်ရန်",
  "rss.created": "RSS feed",
  "rss.updated": "အသစ်ပြင်ထားသော အကြောင်းအရာများ၏ RSS feed",
  "search": "ရှာဖွေရန်",
  "search.config.pipeline": " ",
  "search.placeholder": "ရှာရန်",
  "search.share": "မျှဝေရန်",
  "search.reset": "ရှင်းလင်း",
  "search.result.initializer": "ရှာဖွေမှုကို စတင်နေသည်",
  "search.result.placeholder": "ရှာဖွေရန် စာရိုက်ပါ",
  "search.result.none": "တူညီသော စာရွက်စာတမ်းများ မရှိပါ",
  "search.result.one": "စာရွက်စာတမ်း ၁ ခု တူညီသည်",
  "search.result.other": "စာရွက်စာတမ်း # ခု တူညီသည်",
  "search.result.more.one": "ဤစာမျက်နှာတွင် နောက်ထပ် ၁ ခု",
  "search.result.more.other": "ဤစာမျက်နှာတွင် နောက်ထပ် # ခု",
  "search.result.term.missing": "ပျောက်ဆုံးနေသည်",
  "select.language": "ဘာသာစကား ရွေးချယ်ရန်",
  "select.version": "ဗားရှင်း ရွေးချယ်ရန်",
  "source": "repository သို့ သွားရန်",
  "source.file.contributors": "ပံ့ပိုးသူများ",
  "source.file.date.created": "ဖန်တီးခဲ့သည့်ရက်စွဲ",
  "source.file.date.updated": "နောက်ဆုံး ပြင်ဆင်မှု",
  "tabs": "တက်ဘ်များ",
  "toc": "မာတိကာ",
  "top": "အပေါ်သို့ ပြန်သွားရန်"
}[key] }}{% endmacro %}
