{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "nn",
  "action.edit": "Rediger denne sida",
  "action.skip": "Gå til innhald",
  "action.view": "Vis kjeldekoden for denne sida",
  "announce.dismiss": "Ikkje vis igjen",
  "blog.archive": "Arkiver",
  "blog.categories": "Kategoriar",
  "blog.categories.in": "i",
  "blog.continue": "Les vidare",
  "blog.draft": "Utkast",
  "blog.index": "Tilbake til index",
  "blog.meta": "Metadata",
  "blog.references": "Relaterte lenkjer",
  "clipboard.copy": "Kopier til utklippstavla",
  "clipboard.copied": "Kopiert til utklippstavla",
  "consent.accept": "Akseptert",
  "consent.manage": "Innstillingar",
  "consent.reject": "Avvis",
  "footer": "Footer",
  "footer.next": "Neste",
  "footer.previous": "Førre",
  "header": "Header",
  "meta.comments": "Kommentarar",
  "meta.source": "Kjelde",
  "nav": "Navigasjon",
  "readtime.one": "1 min lesnad",
  "readtime.other": "# min lesnad",
  "rss.created": "RSS feed",
  "rss.updated": "RSS feed for oppdatert innhald",
  "search": "Søk",
  "search.config.lang": "no",
  "search.placeholder": "Søk",
  "search.share": "Del",
  "search.reset": "Nullstill",
  "search.result.initializer": "Startar søk",
  "search.result.placeholder": "Skriv søkjeord",
  "search.result.none": "Ingen treff",
  "search.result.one": "1 treff",
  "search.result.other": "# treff",
  "search.result.more.one": "1 til på denne sida",
  "search.result.more.other": "# fleire på denne sida",
  "search.result.term.missing": "Manglar",
  "select.language": "Vel språk",
  "select.version": "Vel versjon",
  "source": "Gå til kjelde",
  "source.file.contributors": "Bidragsytarar",
  "source.file.date.created": "Oppretta",
  "source.file.date.updated": "Sist oppdatert",
  "tabs": "Faner",
  "toc": "Innhaldsliste",
  "top": "Tilbake til toppen"
}[key] }}{% endmacro %}
