{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "sa",
  "action.edit": "एतत् पृष्ठं सम्पादयतु",
  "action.skip": "सामग्रीं त्यजन्तु",
  "action.view": "अस्य पृष्ठस्य स्रोतः पश्यन्तु",
  "announce.dismiss": "एतत् पुनः न दर्शयतु",
  "blog.archive": "लेखागार",
  "blog.categories": "श्रेणियाँ",
  "blog.categories.in": "इत्यस्मिन्‌",
  "blog.continue": "पठनं निरन्तरं कुर्वन्तु",
  "blog.draft": "प्रारूप",
  "blog.index": "अनुक्रमणिकां प्रति पुनः आगच्छन्तु",
  "blog.meta": "परिदत्तांश",
  "blog.references": "सन्दर्भाः",
  "clipboard.copy": "एतत् प्रतिलिख्यताम्",
  "clipboard.copied": "प्रतिलिपितः भवति",
  "consent.accept": "अहं तत् स्वीकुर्वन् अस्मि",
  "consent.manage": "वविन्यासं प्रबन्धयन्तु",
  "consent.reject": "अहं तत् निराकरोमि",
  "footer": "पादलेखः",
  "footer.next": "अग्रिमः",
  "footer.previous": "पूर्वकृत",
  "header": "शीर्षकम्",
  "meta.comments": "विचाराः",
  "meta.source": "स्रोतः",
  "nav": "मार्गदर्शनम्",
  "readtime.one": "१ निमेषं पठितुं",
  "readtime.other": "# निमेषं पठितुं",
  "rss.created": "आरएसएस सेवा",
  "rss.updated": "आरएसएस सेवातः नवीनतमं अद्यतनम्",
  "search": "अन्वेषण",
  "search.placeholder": "अन्वेषण",
  "search.share": "भजतु",
  "search.reset": "तत् स्वच्छं कुर्वन्तु",
  "search.result.initializer": "अन्वेषणस्य आरम्भः",
  "search.result.placeholder": "अन्वेषणं आरभ्य लिखन्तु",
  "search.result.none": "अभिलेखाः नास्ति",
  "search.result.one": "१ अभिलेखः अस्ति",
  "search.result.other": "# अभिलेखाः सन्ति",
  "search.result.more.one": "अस्मिन् पृष्ठे १ अन्यः अस्ति",
  "search.result.more.other": "अस्मिन् पृष्ठे # अन्ये सन्ति",
  "search.result.term.missing": "शून्य",
  "select.language": "भाषां चिनोतु",
  "select.version": "संस्करणं चिनोतु",
  "source": "भण्डारं गच्छन्तु",
  "source.file.contributors": "अंशदाता",
  "source.file.date.created": "ननिर्माणस्य तिथिः",
  "source.file.date.updated": "परिवर्तनस्य तिथिः",
  "tabs": "पट्टाः",
  "toc": "सामग्रीसारणी",
  "top": "पुनः उपरिभागं प्रति गच्छन्तु"
}[key] }}{% endmacro %}
