{#-
  This file was automatically generated - do not edit
-#}
<!doctype html>
<html>
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width,initial-scale=1">
    <title>{{ config.site_name }}</title>
    <noscript>
      <meta http-equiv="refresh" content="0;url={{ page.meta.location }}">
    </noscript>
    <script>window.location.replace(["{{ page.meta.location }}",window.location.search,window.location.hash].join(""))</script>
  </head>
  <body></body>
</html>
