{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "nl",
  "action.edit": "Wijzig deze pagina",
  "action.skip": "Ga naar inhoud",
  "action.view": "Bron van deze pagina bekijken",
  "announce.dismiss": "Niet meer laten zien",
  "blog.archive": "Archief",
  "blog.categories": "Categorieën",
  "blog.categories.in": "in",
  "blog.continue": "Doorgaan met lezen",
  "blog.draft": "Concept",
  "blog.index": "Terug naar de inhoudsopgave",
  "blog.meta": "Metadata",
  "blog.references": "Gerelateerde links",
  "clipboard.copy": "Kopiëren naar klembord",
  "clipboard.copied": "Gekopieerd naar klembord",
  "consent.accept": "Accepteren",
  "consent.manage": "Instellingen",
  "consent.reject": "Afwijzen",
  "footer": "Footer",
  "footer.next": "Volgende",
  "footer.previous": "Vorige",
  "header": "Header",
  "meta.comments": "Reacties",
  "meta.source": "Bron",
  "nav": "Navigatie",
  "readtime.one": "1 min leestijd",
  "readtime.other": "# min leestijd",
  "rss.created": "RSS feed",
  "rss.updated": "RSS feed met geüpdatet inhoud",
  "search": "Zoeken",
  "search.config.lang": "nl",
  "search.placeholder": "Zoeken",
  "search.share": "Delen",
  "search.reset": "Leegmaken",
  "search.result.initializer": "Zoeken initialiseren",
  "search.result.placeholder": "Typ om te beginnen met zoeken",
  "search.result.none": "Geen overeenkomende documenten",
  "search.result.one": "1 overeenkomende document",
  "search.result.other": "# overeenkomende documenten",
  "search.result.more.one": "1 extra overeenkomst op deze pagina",
  "search.result.more.other": "# extra overeenkomsten op deze pagina",
  "search.result.term.missing": "Ontbreekt",
  "select.language": "Selecteer taal",
  "select.version": "Selecteer versie",
  "source": "Ga naar repository",
  "source.file.contributors": "Bijdragers",
  "source.file.date.created": "Gecreëerd",
  "source.file.date.updated": "Laatst geüpdatet",
  "tabs": "Tabs",
  "toc": "Inhoudsopgave",
  "top": "Terug naar boven"
}[key] }}{% endmacro %}
