{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "eo",
  "action.edit": "Redakti ĉi tiun paĝon",
  "action.skip": "Saltu al enhavo",
  "action.view": "Vidi fonton de ĉi tiu paĝo",
  "announce.dismiss": "Ne montru ĉi tion denove",
  "blog.archive": "Arkivo",
  "blog.categories": "Kategorioj",
  "blog.categories.in": "en",
  "blog.continue": "Daŭrigi legadon",
  "blog.draft": "Malneto",
  "blog.index": "Reen al indekso",
  "blog.meta": "Metadatenoj",
  "blog.references": "Rilataj ligiloj",
  "clipboard.copy": "Kopii al tondujo",
  "clipboard.copied": "Kopiado al klipo",
  "consent.accept": "Akcepti",
  "consent.manage": "Administri agordojn",
  "consent.reject": "Rifuzi",
  "footer": "Piedlinio",
  "footer.next": "Sekva",
  "footer.previous": "Antaŭa",
  "header": "Kaplinio",
  "meta.comments": "Komentoj",
  "meta.source": "Fontkodo",
  "nav": "Navigado",
  "readtime.one": "1 min legado",
  "readtime.other": "# min legado",
  "rss.created": "RSS-fluo",
  "rss.updated": "RSS-fluo de ĝisdatigita enhavo",
  "search": "Serĉi",
  "search.config.lang": "es",
  "search.placeholder": "Serĉo",
  "search.share": "Kunhavigi",
  "search.reset": "Klara",
  "search.result.initializer": "Komencante serĉon",
  "search.result.placeholder": "Tajpu por komenci serĉadon",
  "search.result.none": "Neniuj kongruaj dokumentoj",
  "search.result.one": "1 kongrua dokumento",
  "search.result.other": "# kongruaj dokumentoj",
  "search.result.more.one": "1 pli sur ĉi tiu paĝo",
  "search.result.more.other": "# pli sur ĉi tiu paĝo",
  "search.result.term.missing": "Mankanta",
  "select.language": "Elekti lingvon",
  "select.version": "Elekti version",
  "source": "Iru al deponejo",
  "source.file.contributors": "Kontribuantoj",
  "source.file.date.created": "Kreita",
  "source.file.date.updated": "Lasta ĝisdatigo",
  "tabs": "Langetoj",
  "toc": "Enhavtabelo",
  "top": "Reen al supro"
}[key] }}{% endmacro %}
