{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "bn",
  "action.edit": "এই পেজ এডিট করুন",
  "action.skip": "কনটেন্টে যান",
  "action.view": "পেজের ভিউ",
  "announce.dismiss": "আর কখনো দেখাবে না",
  "blog.archive": "সংরক্ষণাগার",
  "blog.categories": "বিভাগ",
  "blog.categories.in": "বিভাগের মধ্যে",
  "blog.continue": "পড়তে থাকুন",
  "blog.draft": "খসড়া",
  "blog.index": "ইনডেক্সে ফিরে যান",
  "blog.meta": "মেটাডেটা",
  "blog.references": "সম্পর্কিত লিংক",
  "clipboard.copy": "ক্লিপবোর্ডে কপি করুন",
  "clipboard.copied": "ক্লিপবোর্ডে কপি হয়েছে",
  "consent.accept": "গ্রহণ",
  "consent.manage": "সেটিংস ব্যবস্থাপনা",
  "consent.reject": "প্রত্যাখ্যান",
  "footer": "ফুটার",
  "footer.next": "পরে",
  "footer.previous": "পূর্ববর্তী",
  "header": "হেডার",
  "meta.comments": "মন্তব্য",
  "meta.source": "উৎস",
  "nav": "ন্যাভিগেশন",
  "readtime.one": "১ মিনিট পড়া",
  "readtime.other": "# মিনিট পড়া",
  "rss.created": "আরএসএস ফিড",
  "rss.updated": "আপডেট করা বিষয়বস্তুর আরএসএস ফিড",
  "search": "অনুসন্ধান করুন",
  "search.config.pipeline": " ",
  "search.placeholder": "অনুসন্ধান করুন",
  "search.share": "শেয়ার",
  "search.reset": "রিসেট",
  "search.result.initializer": "অনুসন্ধান শুরু করা হচ্ছে",
  "search.result.placeholder": "সার্চ টাইপ করুন",
  "search.result.none": "কিছু পাওয়া যায়নি",
  "search.result.one": "১ টা ডকুমেন্ট",
  "search.result.other": "# টা ডকুমেন্ট",
  "search.result.more.one": "এই পৃষ্ঠায় আরও ১টি আছে",
  "search.result.more.other": "এই পৃষ্ঠায় আরও #টি আছে",
  "search.result.term.missing": "অনুপস্থিত",
  "select.language": "ভাষা নির্বাচন করুণ",
  "select.version": "সংস্করণ নির্বাচন করুণ",
  "source": "রিপোজিটরিতে যান",
  "source.file.contributors": "অবদানকারী",
  "source.file.date.created": "তৈরি হয়েছে",
  "source.file.date.updated": "শেষ আপডেট",
  "tabs": "ট্যাব",
  "toc": "সূচি তালিকা",
  "top": "উপরে ফিরে যাও"
}[key] }}{% endmacro %}
