{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "eu",
  "action.edit": "Editatu orri hau",
  "action.skip": "Joan zuzenean edukira",
  "action.view": "Ikusi orri honen iturburua",
  "announce.dismiss": "Ez erakutsi hau berriro",
  "blog.archive": "Artxiboa",
  "blog.categories": "Kategoriak",
  "blog.categories.in": "kategoria",
  "blog.continue": "Jarraitu irakurtzen",
  "blog.draft": "Zirriborroa",
  "blog.index": "Itzuli aurkibidera",
  "blog.meta": "Metadatuak",
  "blog.references": "Erlazionatutako estekak",
  "clipboard.copy": "Kopiatu arbelean",
  "clipboard.copied": "Arbelean kopiatuta",
  "consent.accept": "Onartu",
  "consent.manage": "Kudeatu ezarpenak",
  "consent.reject": "Ukatu",
  "footer": "Orri-oina",
  "footer.next": "Hurrengoa",
  "footer.previous": "Aurrekoa",
  "header": "Atalburua",
  "meta.comments": "Iruzkinak",
  "meta.source": "Iturburua",
  "nav": "Nabigazioa",
  "readtime.one": "Minutu batean irakurtzeko",
  "readtime.other": "# minututan irakurtzeko",
  "rss.created": "RSS jarioa",
  "rss.updated": "Eduki eguneratuen RSS jarioa",
  "search": "Bilatu",
  "search.placeholder": "Bilatu",
  "search.share": "Partekatu",
  "search.reset": "Garbitu",
  "search.result.initializer": "Bilaketa hasieratzen",
  "search.result.placeholder": "Idatzi bilatzen hasteko",
  "search.result.none": "Bat datorren dokumenturik ez",
  "search.result.one": "Bat datorren dokumentu bat",
  "search.result.other": "Bat datozen # dokumentu",
  "search.result.more.one": "Bat gehiago orri honetan",
  "search.result.more.other": "# gehiago orri honetan",
  "search.result.term.missing": "Falta da",
  "select.language": "Hautatu hizkuntza",
  "select.version": "Hautatu bertsioa",
  "source": "Joan biltegira",
  "source.file.contributors": "Kolaboratzaileak",
  "source.file.date.created": "Sortze data",
  "source.file.date.updated": "Azken eguneratzea",
  "tabs": "Fitxak",
  "toc": "Edukiak",
  "top": "Igo goraino"
}[key] }}{% endmacro %}
