{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "hu",
  "action.edit": "Oldal szerkesztése",
  "action.skip": "Kihagyás",
  "action.view": "Oldal forrásának megtekintése",
  "announce.dismiss": "Ne mutasd többet",
  "blog.archive": "Archívum",
  "blog.categories": "Kategóriák",
  "blog.categories.in": "Kategória:",
  "blog.continue": "Folytatás",
  "blog.draft": "Piszkozat",
  "blog.index": "Vissza a főoldalra",
  "blog.meta": "Metaadat",
  "blog.references": "Kapcsolódó linkek",
  "clipboard.copy": "Másolás vágólapra",
  "clipboard.copied": "Vágólapra másolva",
  "consent.accept": "Elfogadás",
  "consent.manage": "Beállítások",
  "consent.reject": "Visszautasítás",
  "footer": "Élőláb",
  "footer.next": "Következő",
  "footer.previous": "Előző",
  "header": "Élőfej",
  "meta.comments": "Hozzászólások",
  "meta.source": "Forrás",
  "nav": "Navigáció",
  "readtime.one": "1 percnyi",
  "readtime.other": "# percnyi",
  "rss.created": "RSS feed",
  "rss.updated": "Frissített tartalom RSS feedje",
  "search": "Keresés",
  "search.config.lang": "hu",
  "search.placeholder": "Keresés",
  "search.share": "Megosztás",
  "search.reset": "Törlés",
  "search.result.initializer": "Keresés inicializálása",
  "search.result.placeholder": "Kereséshez írj ide valamit",
  "search.result.none": "Nincs találat",
  "search.result.one": "1 egyező dokumentum",
  "search.result.other": "# egyező dokumentum",
  "search.result.more.one": "1 további találat az oldalon",
  "search.result.more.other": "# további találat az oldalon",
  "search.result.term.missing": "Üres",
  "select.language": "Nyelvváltás",
  "select.version": "Verzióváltás",
  "source": "Főoldalra ugrás",
  "source.file.contributors": "Szerzők",
  "source.file.date.created": "Létrehozva",
  "source.file.date.updated": "Utolsó frissítés",
  "tabs": "Lapok",
  "toc": "Tartalomjegyzék",
  "top": "Vissza a tetejére"
}[key] }}{% endmacro %}
