{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "ca",
  "action.edit": "Edita aquesta pàgina",
  "action.skip": "Salta el contingut",
  "action.view": "Visualitza el codi font",
  "announce.dismiss": "No ho tornis a mostrar",
  "blog.archive": "Arxiva",
  "blog.categories": "Categories",
  "blog.categories.in": "a",
  "blog.continue": "Continua llegint",
  "blog.draft": "Esborrany",
  "blog.index": "Torna a l'índex",
  "blog.meta": "Metadades",
  "blog.references": "Enllaços relacionats",
  "clipboard.copy": "Còpia al porta-retalls",
  "clipboard.copied": "Copiat al porta-retalls",
  "consent.accept": "Accepta",
  "consent.manage": "Gestiona la configuració",
  "consent.reject": "Rebutja",
  "footer": "Peu de pàgina",
  "footer.next": "Següent",
  "footer.previous": "Anterior",
  "header": "Capçalera",
  "meta.comments": "Comentaris",
  "meta.source": "Codi font",
  "nav": "Navegació",
  "readtime.one": "1 min de lectura",
  "readtime.other": "# min de lectura",
  "rss.created": "Canal RSS",
  "rss.updated": "Canal RSS de contingut actualitzat",
  "search": "Cerca",
  "search.placeholder": "Cerca",
  "search.share": "Comparteix",
  "search.reset": "Neteja",
  "search.result.initializer": "Inicialitzant cerca",
  "search.result.placeholder": "Escriu per a començar a cercar",
  "search.result.none": "Cap document coincideix",
  "search.result.one": "1 document coincident",
  "search.result.other": "# documents coincidents",
  "search.result.more.one": "1 més en aquesta pàgina",
  "search.result.more.other": "# més en aquesta pàgina",
  "search.result.term.missing": "Desaparegut",
  "select.language": "Selecciona la llengua",
  "select.version": "Selecciona la versió",
  "source": "Ves al repositori",
  "source.file.contributors": "Col·laboradors",
  "source.file.date.created": "Creada",
  "source.file.date.updated": "Darrera actualització",
  "tabs": "Pestanyes",
  "toc": "Taula de continguts",
  "top": "Torna a l'inici"
}[key] }}{% endmacro %}
