{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "ku",
  "direction": "rtl",
  "action.edit": "دەستکاری ئەم پەڕە بکە",
  "action.skip": "ئەم ناوەڕۆکە بپەڕێنە",
  "action.view": "سەرچاوەی ئەم لاپەڕەیە نیشان بدە",
  "announce.dismiss": "دووبارە ئەمە پیشان مەدە",
  "clipboard.copy": "لەبەرگتنەوە بۆ کلیپبۆرد",
  "clipboard.copied": "لەبەرگیرایەوە بۆ کلیپ بۆرد",
  "consent.accept": "ڕازیبوون",
  "consent.manage": "بەڕیوەبردنی ڕیکخستنەکان",
  "consent.reject": "ڕەتکردنەوە",
  "footer": "ژێرپەڕە",
  "footer.next": "دواتر",
  "footer.previous": "پێشتر",
  "header": "ناونیشانی بەڕه",
  "meta.comments": "لێدوانەکان",
  "meta.source": "سەرجاوە",
  "nav": "ڕێنیشاندەر",
  "search": "گەڕان",
  "search.config.pipeline": " ",
  "search.placeholder": "گەڕان",
  "search.share": "گەڕان",
  "search.reset": "سڕینەوە",
  "search.result.initializer": "ئامادەکردنی گەڕان",
  "search.result.placeholder": "بنووسە بۆ دەستپێکردن بە گەڕان",
  "search.result.none": "هیچ بەڵگەنامەیەکی هاوتا نیە",
  "search.result.one": "١ بەڵگەنامەی هاوتا",
  "search.result.other": "بەڵگەنامەی هاوتا #",
  "search.result.more.one": "١ دانەی تر لەسەر ئەم پەڕەیە",
  "search.result.more.other": "دانەی تر لەسەر ئەم پەڕەیە #",
  "search.result.term.missing": "ونبوو",
  "select.language": "زمان دیاریبکە",
  "select.version": "وەشان دیاریبکە",
  "source": "بڕۆ بۆ کۆگا",
  "source.file.date.created": "دروسکت کرا",
  "source.file.date.updated": "دوایین نوێکردنەوە",
  "tabs": "تابەکان",
  "toc": "خشتەی ناوەڕۆکەکان",
  "top": "گەڕانەوە بۆ سەرەوە"
}[key] }}{% endmacro %}
