{#-
  This file was automatically generated - do not edit
-#}
{% macro t(key) %}{{ {
  "language": "ka",
  "action.edit": "გვერდის რედაქტირება",
  "action.skip": "შიგთავსზე გადასვლა",
  "action.view": "გვერდის პირველწყაროს ნახვა",
  "announce.dismiss": "მეტად აღარ მაჩვენო",
  "blog.archive": "არქივი",
  "blog.categories": "კატეგორიები",
  "blog.continue": "წაკითხვის გაგრძელება",
  "blog.draft": "შაბლონი",
  "blog.index": "ინდექსზე დაბრუნება",
  "blog.meta": "მეტამონაცემები",
  "blog.references": "დაკავშირებული ბმულები",
  "clipboard.copy": "კოპირება",
  "clipboard.copied": "კოპირებულია",
  "consent.accept": "დათანხმება",
  "consent.manage": "პარამეტრების მართვა",
  "consent.reject": "უარყოფა",
  "footer.next": "შემდეგი",
  "footer.previous": "წინა",
  "meta.comments": "კომენტარები",
  "meta.source": "წყარო",
  "nav": "ნავიგაცია",
  "readtime.one": "1 წუთში წაკითხვადი",
  "search": "ძებნა",
  "search.config.pipeline": " ",
  "search.placeholder": "ძებნა",
  "search.share": "გაზიარება",
  "search.reset": "გასუფთავება",
  "search.result.placeholder": "ჩაწერე ძებნის დასაწყებად",
  "search.result.none": "დოკუმენტი ვერ მოიძებნა",
  "search.result.one": "მოიძებნა 1 დოკუმენტი",
  "search.result.other": "მოიძებნა # დოკუმენტი",
  "search.result.more.one": "კიდევ 1 ამ გვერდზე",
  "search.result.more.other": "კიდევ # ამ გვერდზე",
  "select.language": "ენის არჩევა",
  "select.version": "ვერსიის არჩევა",
  "source": "საცავში გადასვლა",
  "source.file.contributors": "წვლილის შემტანები",
  "source.file.date.created": "შეიქმნა",
  "source.file.date.updated": "ბოლო განახლება",
  "tabs": "ტაბები",
  "toc": "სარჩევი",
  "top": "თავფურცელზე დაბრუნება"
}[key] }}{% endmacro %}
